function [edges_3d,G1S,G2S,G3S] = Canny3Dedge(IMSS,sigma)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [edges_3d,G1S,G2S,G3S] = Canny3Dedge(IMSS,sigma)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: This is a 3D edge detection algorithm using
% the Canny algorithm.
% Input  IMSS is a 3D array containing the data: 
%             intensity magnitudes at each voxel.
%       sigma is the standard deviation used
%             for Gaussian smoothing of images
% Output G1S,G2S,G3S are the estimated horizontal, vertical,
%             and time derivatives of the intensity at each voxel.
% Output edges_3d contains the estimated squared-magnitude of the
%             gradient at each voxel, normalized to have max = 1.
% David A. Schug, Glenn R. Easley, Dianne P. O'Leary Fall 2012
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set some options. These could be made optional 
% arguments of the function.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% size of filter

sz =3;

% dimensions of convolution filters

hsize = [sz sz sz]; 

% setting to normalize gaussian or not

normalize = 1; 

%% Initialize the gaussian derivatives.

G1D = ndgauss(hsize,sigma*[1 1 1],'der',[1,0,0],'normalize',normalize);
G2D = ndgauss(hsize,sigma*[1 1 1],'der',[0,1,0],'normalize',normalize);
G3D = ndgauss(hsize,sigma*[1 1 1],'der',[0,0,1],'normalize',normalize);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Begin the edge detection algorithm.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Determine dimensions of image sequence

% and pad the boundaries.

[~, ~, n_ist] = size(IMSS);

IMSSex(:,:,1)=squeeze(IMSS(:,:,1));
IMSSex(:,:,2)=squeeze(IMSS(:,:,1));

IMSSex(:,:,3:n_ist+2)=IMSS(:,:,1:n_ist);

IMSSex(:,:,n_ist+3)=squeeze(IMSS(:,:,n_ist));
IMSSex(:,:,n_ist+4)=squeeze(IMSS(:,:,n_ist));

IMSS=IMSSex;

% Smooth the image sequence at a standard deviation sigma.

IMSSs = smooth3(IMSS, 'gaussian', sz, sigma);

% Estimate the horizontal, vertical, and time derivatives 
% of the data using G1, G2, and G3.           

G1S = conv3(IMSSs,G1D);
G2S = conv3(IMSSs,G2D);
G3S = conv3(IMSSs,G3D);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Trim the padded horizontal, vertical, and time 
% derivatives.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

G1S = G1S(:,:,3:n_ist+2);
G2S = G2S(:,:,3:n_ist+2);
G3S = G3S(:,:,3:n_ist+2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the squared-magnitude of the gradient at each 
% voxel and return it in edges_3d.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

edges_3d = G1S.^2 + G2S.^2 + G3S.^2;

edges_3d = edges_3d / max(edges_3d(:));
