function [edges_3d,G1,G2,G3] = Wavelet3Dedge(IMSS,n_levels)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [edges_3d,G1,G2,G3] = Wavelet3Dedge(IMSS,n_levels)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: This is a 3D edge detection algorithm using wavelets.
% Input  IMSS is a 3D array containing the data:
%             intensity magnitudes at each voxel.
%        n_levels is the number of levels in the wavelet decomposition
%                 (default n_levels = 3)
% Output G1,G2,G3 are the estimated horizontal, vertical,
%             and time derivatives of the intensity at each voxel.
% Output edges_3d contains the estimated squared-magnitude of the
%             gradient at each voxel, normalized to max 1.
% David Schug, Glenn R. Easley, Dianne P. O'Leary Fall 2012
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set the number of levels in the algorithm.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (nargin < 2)
    n_levels = 3;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate smoothing and derivative masks
% for convolution.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Smoothing operator:

v = [ 1  2  1;
      2  4  2;
      1  2  1];

c= [ 1  1  1;
     1  2  1;
     1  1  1];

S3D(:,:,1) = c;
S3D(:,:,2) = v;
S3D(:,:,3) = c;
S3D = S3D / sum(abs(S3D(:)));

% Time derivative:

d3(:,:,1) = v;
d3(:,:,2) = zeros(3);
d3(:,:,3) = -v;
d3 = d3 / sum(abs(d3(:)));

% Horizontal derivative:

d1 = permute(d3,[2 3 1]);

% Vertical derivative:

d2 = permute(d3,[3 1 2]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Begin the edge detection algorithm.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Determine dimensions of image sequence
% and pad the boundaries in the time dimension.

npad = 4;

[~,~,n_ist_in] = size(IMSS);

IMSSex(:,:,1) = squeeze(IMSS(:,:,1));
IMSSex(:,:,2) = squeeze(IMSS(:,:,1));

IMSSex(:,:,3:n_ist_in+2) = IMSS(:,:,1:n_ist_in);

IMSSex(:,:,n_ist_in+3) = squeeze(IMSS(:,:,n_ist_in));
IMSSex(:,:,n_ist_in+4) = squeeze(IMSS(:,:,n_ist_in));

IMSS=IMSSex;

% Estimate the horizontal, vertical, and time derivatives
% of the data using d1, d2, and d3.

G1 = conv3(IMSS,d1);
G2 = conv3(IMSS,d2);
G3 = conv3(IMSS,d3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Begin the loop through the wavelet levels 1,..., n_levels.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for level = 1 : n_levels
    
    % Perform wavelet filtering.
    
    G1s = conv3(G1,S3D);
    G2s = conv3(G2,S3D);
    G3s = conv3(G3,S3D);
    
    % Add wavelet increments to G1, G2, and G3.
    
    G1=G1.*(abs(G1)<=abs(G1s))+G1s.*(abs(G1)>abs(G1s));
    G2=G2.*(abs(G2)<=abs(G2s))+G2s.*(abs(G2)>abs(G2s));
    G3=G3.*(abs(G3)<=abs(G3s))+G3s.*(abs(G3)>abs(G3s));
    
end  % for level = 1 : n_levels

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The loop through levels is complete.
% Trim the padded horizontal, vertical, and time 
% derivatives.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

G1 = G1(:,:,3:n_ist_in+2);
G2 = G2(:,:,3:n_ist_in+2);
G3 = G3(:,:,3:n_ist_in+2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Zero the boundary edge pixels.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

G1(1,:,:) = 0;
G1(:,1,:) = 0;
G1(:,:,1) = 0;

G2(1,:,:) = 0;
G2(:,1,:) = 0;
G2(:,:,1) = 0;

G3(1,:,:) = 0;
G3(:,1,:) = 0;
G3(:,:,1) = 0;

G1(end,:,:) = 0;
G1(:,end,:) = 0;
G1(:,:,end) = 0;

G2(end,:,:) = 0;
G2(:,end,:) = 0;
G2(:,:,end) = 0;

G3(end,:,:) = 0;
G3(:,end,:) = 0;
G3(:,:,end) = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the squared-magnitude of the gradient at each 
% voxel and return it in edges_3d.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

edges_3d = G1.^2 + G2.^2 + G3.^2;

edges_3d = edges_3d / max(edges_3d(:));
